@@ -4,7 +4,7 @@
 
 from __future__ import annotations
 
-INCLUDES = """
+INCLUDES = r"""
 /* define our OpenSSL API compatibility level to 1.1.0. Any symbols older than
    that will raise an error during compilation. */
 #define OPENSSL_API_COMPAT 0x10100000L
@@ -42,37 +42,19 @@ INCLUDES = """
 #define CRYPTOGRAPHY_IS_BORINGSSL 0
 #endif
 
-#if OPENSSL_VERSION_NUMBER < 0x10101040
-    #error "pyca/cryptography MUST be linked with Openssl 1.1.1d or later"
+#if defined(OPENSSL_IS_AWSLC)
+#define CRYPTOGRAPHY_IS_AWSLC 1
+#else
+#define CRYPTOGRAPHY_IS_AWSLC 0
 #endif
 
-#define CRYPTOGRAPHY_OPENSSL_300_OR_GREATER \
-    (OPENSSL_VERSION_NUMBER >= 0x30000000 && !CRYPTOGRAPHY_IS_LIBRESSL)
-#define CRYPTOGRAPHY_OPENSSL_320_OR_GREATER \
-    (OPENSSL_VERSION_NUMBER >= 0x30200000 && !CRYPTOGRAPHY_IS_LIBRESSL)
 
-#define CRYPTOGRAPHY_OPENSSL_LESS_THAN_111E \
-    (OPENSSL_VERSION_NUMBER < 0x10101050 || CRYPTOGRAPHY_IS_LIBRESSL)
-
-#if CRYPTOGRAPHY_IS_LIBRESSL
-#define CRYPTOGRAPHY_LIBRESSL_LESS_THAN_380 \
-    (LIBRESSL_VERSION_NUMBER < 0x3080000f)
-
-#else
-#define CRYPTOGRAPHY_LIBRESSL_LESS_THAN_380 (0)
+#if OPENSSL_VERSION_NUMBER < 0x10101050
+    #error "pyca/cryptography MUST be linked with Openssl 1.1.1e or later"
 #endif
 """
 
 TYPES = """
-static const int CRYPTOGRAPHY_OPENSSL_300_OR_GREATER;
-static const int CRYPTOGRAPHY_OPENSSL_320_OR_GREATER;
-
-static const int CRYPTOGRAPHY_OPENSSL_LESS_THAN_111E;
-
-static const int CRYPTOGRAPHY_LIBRESSL_LESS_THAN_380;
-
-static const int CRYPTOGRAPHY_IS_LIBRESSL;
-static const int CRYPTOGRAPHY_IS_BORINGSSL;
 """
 
 FUNCTIONS = """
